# ===============================================================================
# Copyright 2021 ross
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ===============================================================================
import os

import click
import subprocess
import os
import platform


def yes(msg):
    if not msg.endswith(" "):
        msg = "{} ".format(msg)

    return input(msg) in ("", "y", "yes", "Yes", "YES")


def r_mkdir(p, *args):
    p = os.path.join(p, *args)
    if p and not os.path.isdir(p):
        try:
            os.mkdir(p)
        except OSError:
            r_mkdir(os.path.dirname(p))
            os.mkdir(p)
    return p


def make_dir(root, name):
    for d in (root, os.path.join(root, name)):
        if not os.path.isdir(d):
            os.mkdir(d)


def write(p, t, overwrite=False, verbose=False):
    if not os.path.isfile(p) or overwrite:
        click.echo(f"wrote file: {p}")
        if verbose:
            click.secho(f"{p} contents: ==============", fg="blue")
            click.secho(t, fg="yellow", bg="black")
            click.secho(f"{p} end: ================================", fg="blue")

        head, tail = os.path.split(p)
        r_mkdir(head)
        if t is None:
            click.secho("writing an empty file", fg="yellow")

        with open(p, "w") as wfile:
            wfile.write(t or "")
    else:
        click.secho(f"file already exists skipping: {p}", fg="red")


def echo_config(*args):
    click.secho("------------ Configuration -------------", fg="yellow")
    for a in args:
        click.secho(f"={a}", fg="yellow")
    click.secho("------------ Configuration End -------------", fg="yellow")


def is_tool(name):
    try:
        devnull = open(os.devnull)
        subprocess.Popen([name], stdout=devnull, stderr=devnull).communicate()
    except OSError as e:
        if e.errno == os.errno.ENOENT:
            return False
    return True


def find_prog(prog):
    if is_tool(prog):
        cmd = "where" if platform.system() == "Windows" else "which"
        out = subprocess.check_output([cmd, prog])
        if out is not None:
            return out.decode().strip()


def r_mkdir(p):
    if p and not os.path.isdir(p):
        try:
            os.mkdir(p)
        except OSError:
            r_mkdir(os.path.dirname(p))
            os.mkdir(p)


def handle_check_call(*args, **kw):
    try:
        subprocess.check_call(*args, **kw)
    except subprocess.CalledProcessError:
        import traceback

        exc = traceback.format_exc()
        click.secho(exc, fg="red")


# ============= EOF =============================================



# ===============================================================================
# Copyright 2021 ross
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ===============================================================================


import os
import shutil
import subprocess
import platform

import click
import yaml
from git import Repo

from pcm import util, requirements, render
from pcm.util import find_prog, handle_check_call, r_mkdir

IS_MAC = platform.system() == "Darwin"
IS_WINDOWS = platform.system() == "Windows"
HOME = os.path.expanduser("~")
EDM_ENVS_ROOT = os.path.join(HOME, ".edm", "envs")
EDM_BIN = os.path.join(EDM_ENVS_ROOT, "edm", "bin")


def _login(env, app_id):
    # write the user
    root = os.path.join(HOME, f".pychron.{app_id}")
    user_file = os.path.join(root, "users.yaml")
    t = {
        "users": [
            "root",
        ],
        "last_login": "root",
    }
    util.write(user_file, yaml.dump(t))

    environment_file = os.path.join(root, "environments.yaml")
    pe = os.path.join(HOME, env)
    t = {
        "env": pe,
        "envs": [
            pe,
        ],
    }
    util.write(environment_file, yaml.dump(t))


def _edm(environment, app, verbose):
    click.secho("edm install", bold=True, fg="green")
    req = requirements.EDM_REQUIREMENTS
    pip_req = requirements.PIP_REQUIREMENTS

    if app == "pyvalve":
        req.extend(requirements.VALVE_REQUIREMENTS)
    else:
        pip_req.extend(requirements.PIP_EXTRAS)

    cmdargs = ["edm", "install", "-y"] + req
    active_python = os.path.join(HOME, ".edm")
    if environment:
        active_python = os.path.join(
            active_python, "envs", environment, "bin", "python"
        )
        cmdargs.extend(["--environment", environment])

        handle_check_call(["edm", "environments", "create", environment])
    else:
        active_python = os.path.join(active_python, "bin", "python")

    if verbose:
        click.echo(f'requirements: {" ".join(req)}')
        click.echo(f'command: {" ".join(cmdargs)}')

    handle_check_call(cmdargs)
    handle_check_call(
        [
            active_python,
            "-m",
            "pip",
            "install",
            "--no-dependencies",
        ]
        + pip_req
    )


def _render_template(dargs, name, overwrite):
    d = util.r_mkdir(*dargs)
    p = os.path.join(d, name)
    util.write(p, render.render_template(name), overwrite)


def _scripts(env, use_ngx, overwrite, verbose):
    root = os.path.join(HOME, env)
    _render_template((root, "scripts"), "defaults.yaml", overwrite)

    measurement_args = "measurement", "unknown"
    extraction_args = "extraction", "extraction"
    procedure_args = "procedures", "procedure"
    post_eq_args = "post_equilibration", "post_equilibration"
    post_m_args = "post_measurement", "post_measurement"

    for name, filename in (
        measurement_args,
        extraction_args,
        procedure_args,
        post_eq_args,
        post_m_args,
    ):
        _render_template(
            (root, "scripts", name), "example_{}.py".format(filename), overwrite
        )

    for dname, pname in (("fits", "nominal.yaml"), ("hops", "hops.yaml")):
        _render_template((root, "scripts", "measurement", dname), pname, overwrite)


def _setupfiles(env, use_ngx, overwrite, verbose):
    root = os.path.join(HOME, env)

    sf = util.r_mkdir(root, "setupfiles")

    for d, ps, enabled in (
        ("canvas2D", ("canvas.yaml", "canvas_config.xml", "alt_config.xml"), True),
        ("extractionline", ("valves.yaml",), True),
        ("monitors", ("system_monitor.cfg",), True),
        (
            "devices",
            (
                "ngx_switch_controller.cfg",
                "spectrometer_microcontroller.cfg",
                "NGXGPActuator.cfg",
            ),
            use_ngx,
        ),
        ("", ("startup_tests.yaml", "experiment_defaults.yaml"), True),
    ):
        if d:
            # out = os.path.join(sf, d)
            # util.make_dir(sf, d)
            out = util.r_mkdir(sf, d)
        else:
            out = sf

        for template in ps:
            txt = render.render_template(template)
            if template == "valves.yaml" and use_ngx:
                txt += """- name: MS_Inlet
                address: PIV
                """
            p = os.path.join(out, template)
            util.write(p, txt, overwrite, verbose)

    if use_ngx:
        _spectrometer_init("ngx", env, overwrite)


def _code(fork, branch, app_id):
    update_root = os.path.join(HOME, f".pychron.{app_id}")
    ppath = os.path.join(update_root, "pychron")
    # locate the git executable
    git = find_prog("git")
    if not git:
        click.secho("Could not locate git executable", fg="red")
        return

    if not os.path.isdir(update_root):
        os.mkdir(update_root)

    if os.path.isdir(ppath):
        if not util.yes(
            "Pychron source code already exists. Remove and re-clone [y]/n"
        ):
            shutil.rmtree(ppath)

    url = f"https://github.com/{fork}/pychron.git"

    subprocess.call([git, "clone", url, f"--branch={branch}", ppath])
    subprocess.call([git, "status"], cwd=ppath)


def _launcher(
    conda, environment, app, org, app_id, login, msv, output, overwrite, verbose
):
    click.echo("launcher")

    if IS_MAC:
        if conda:
            template = "launcher_mac_conda"
        else:
            template = "launcher_mac"
    else:
        template = "launcher_pc"
        output = "pychron_launcher.bat"

    ctx = {
        "github_org": org,
        "app_name": app,
        "app_id": app_id,
        "use_login": login,
        "massspec_db_version": msv,
        "edm_envs_root": EDM_ENVS_ROOT,
        "edm_env": environment,
        "pychron_path": os.path.join(HOME, f".pychron.{app_id}", "pychron"),
    }

    txt = render.render_template(template, **ctx)

    if output is None:
        output = "pychron_launcher.sh"

    if verbose:
        click.echo(f"Writing launcher script: {output}")
        click.echo(txt)
    util.write(output, txt, overwrite)

    if not IS_WINDOWS:
        # make launcher executable
        subprocess.call(["chmod", "+x", output])


def _email(env, overwrite):
    # copy the credentials file to appdata
    click.echo("make initialization file")
    template = "credentials.json"
    txt = render.render_template(template)
    root = os.path.join(HOME, env)
    sf = ".appdata"
    util.make_dir(root, sf)
    p = os.path.join(root, sf, template)
    util.write(p, txt, overwrite=overwrite)


def _init(env, org, use_ngx, overwrite, verbose):
    click.echo("make initialization file")
    template = "initialization.xml"
    txt = render.render_template(template)
    if verbose:
        click.echo("======== Initialization.xml contents start ========")
        click.echo(txt)
        click.echo("======== Initialization.xml contents end ========")

    root = os.path.join(HOME, env)

    d = util.r_mkdir(root, "setupfiles")

    p = os.path.join(d, "initialization.xml")
    util.write(p, txt, overwrite=overwrite)

    d = util.r_mkdir(root, "preferences")
    gctx = dict(general_organization=org, general_remote="{}/Laboratory")
    uctx = dict(
        build_repo=os.path.join(HOME, ".pychron.0", "pychron"),
        build_remote="PychronLabsLLC/pychron",
        build_branch="dev/dr",
    )

    sf = os.path.join(HOME, env, "setupfiles")
    ectx = {
        "canvas_path": os.path.join(sf, "canvas2D", "canvas.yaml"),
        "canvas_config_path": os.path.join(sf, "canvas2D", "canvas_config.xml"),
        "valves_path": os.path.join(sf, "extractionline", "valves.yaml"),
    }

    for template, ctx, flag in (
        ("general.ini", gctx, True),
        ("dvc.ini", {}, True),
        ("update.ini", uctx, True),
        ("arar_constants.ini", {}, True),
        ("extractionline.ini", ectx, True),
        ("ngx.ini", {}, use_ngx),
    ):
        if flag:
            txt = render.render_template(template, **ctx)
            p = os.path.join(d, template)
            util.write(p, txt, overwrite=overwrite)


def _makefile(name, env, overwrite):
    txt = render.render_template(name)
    p = os.path.join(HOME, env, name)
    util.write(p, txt, overwrite=overwrite)


def _spectrometer_init(kind, env, overwrite):
    kind = kind.lower()
    root = os.path.join(HOME, env)

    # mftable
    txt = render.render_template(f"{kind}_mftable.csv")
    p = os.path.join(root, "setupfiles", "spectrometer", "mftables", "mftable.csv")
    util.write(p, txt, overwrite)

    # config
    txt = render.render_template(f"{kind}_config.cfg")
    p = os.path.join(root, "setupfiles", "spectrometer", "configurations", "config.cfg")
    util.write(p, txt, overwrite)

    # detectors
    txt = render.render_template(f"{kind}_detectors.yaml")
    p = os.path.join(root, "setupfiles", "spectrometer", "detectors.yaml")
    util.write(p, txt, overwrite)

    if kind == "ngx":
        # preferences
        template = "ngx.ini"
        txt = render.render_template(template)
        p = os.path.join(root, "preferences", template)
        util.write(p, txt, overwrite=overwrite)


def _metarepo(name, env, overwrite):
    root = os.path.join(HOME, env, "data", ".dvc", name)
    r_mkdir(root)
    if not os.path.isfile(os.path.join(root, ".git")):
        repo = Repo.init(root)
    else:
        repo = Repo(root)

    ih = os.path.join(root, "irradiation_holders")
    r_mkdir(ih)
    template = "Grid.txt"
    p = os.path.join(ih, template)
    txt = render.render_template(template)
    util.write(p, txt, overwrite=overwrite)
    repo.index.add(p)

    noi = os.path.join(root, "NoIrradiation")
    r_mkdir(noi)

    template = "A.json"
    p = os.path.join(noi, template)
    txt = render.render_template(template)
    util.write(p, txt, overwrite=overwrite)

    prod = os.path.join(noi, "productions")
    r_mkdir(prod)
    template = "NoIrradiation.json"
    p = os.path.join(prod, template)
    txt = render.render_template(template)
    util.write(p, txt, overwrite=overwrite)

    p = os.path.join(noi, "productions.json")
    txt = '{"A"; "NoIrradiation"}'
    util.write(p, txt, overwrite=overwrite)

    p = os.path.join(noi, "chronology.txt")
    txt = ""
    util.write(p, txt, overwrite=overwrite)


# ============= EOF =============================================



# ===============================================================================
# Copyright 2021 ross
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ===============================================================================
import os

import click
import platform

from pcm.func import (
    _login,
    _edm,
    _setupfiles,
    _code,
    _launcher,
    _init,
    _email,
    _scripts,
    _makefile,
    _spectrometer_init,
    _metarepo,
)
from pcm.util import echo_config


@click.group()
def cli():
    pass


@cli.command()
@click.option(
    "--template",
    default=None,
    help="Device Template to use. Typically the device models name",
)
@click.argument("name")
def device(template, name):
    click.echo("Create a new device configuration")


@cli.command()
@click.option(
    "--app",
    default="pycrunch",
    help="Application style to install. pycrunch, pyexperiment,...",
)
@click.option("--conda/--no-conda", default=False, help="Use Conda")
@click.option("--src/--no-src", "use_src", default=True, help="install the source code")
@click.option("--app_id", default=0, help="set the app id")
@click.option(
    "--fork", default="PychronLabsLLC", help="Name of the pychron fork to clone"
)
@click.option(
    "--org",
    default="NMGRL",
    help="Github organization for storing laboratory files such as Plot Options",
)
@click.option("--branch", default="dev/dr", help="Name of the pychron fork to clone")
@click.option(
    "--setupfiles/--no-setupfiles",
    "use_setupfiles",
    default=False,
    help="Install pychron setupfiles",
)
@click.option(
    "--init/--no-init",
    "use_init",
    default=True,
    help="Install pychron initalization files",
)
@click.option("--env", default="Pychron", help="Environment, aka root directory name")
@click.option(
    "--edm/--no-edm", "use_edm", default=True, help="Install the EDM environment"
)
@click.option("--environment", default="pychron", help="Name of the EDM environment")
@click.option(
    "--launcher/--no-launcher",
    "use_launcher",
    default=True,
    help="make a launcher script",
)
@click.option("--app_id", default=0, help="set the app id")
@click.option(
    "--use_login/--no-use_login", default=True, help="Write default login files"
)
@click.option("--login/--no-login", default=0, help="show login window at startup")
@click.option(
    "--massspec_db_version", "msv", default=16, help="massspec database version"
)
@click.option("--use_ngx/--no-ngx", default=False, help="Install NGX")
@click.option(
    "--overwrite/--no-overwrite", default=False, help="Overwrite the file if it exists"
)
@click.option("--verbose/--no-verbose", default=False, help="Verbose output")
def wizard(
    app,
    conda,
    use_src,
    app_id,
    fork,
    org,
    branch,
    use_setupfiles,
    use_init,
    env,
    use_edm,
    environment,
    use_launcher,
    use_login,
    login,
    msv,
    overwrite,
    use_ngx,
    verbose,
):
    echo_config(
        app,
        conda,
        use_src,
        app_id,
        fork,
        org,
        branch,
        use_setupfiles,
        use_init,
        env,
        use_edm,
        environment,
        use_launcher,
        use_login,
        login,
        msv,
        overwrite,
        use_ngx,
        verbose,
    )
    click.secho("Install the pychron application", bold="True", fg="green")
    if use_src:
        _code(fork, branch, app_id)

    if use_init:
        _init(env, org, use_ngx, overwrite, verbose)

    if use_setupfiles:
        _setupfiles(env, use_ngx, overwrite, verbose)

    if use_edm:
        _edm(environment, app, verbose)

    if use_launcher:
        _launcher(
            conda, environment, app, fork, app_id, login, msv, None, overwrite, verbose
        )

    if use_login:
        _login(env, app_id)


@cli.command()
@click.option("--env", default="Pychron", help="Environment, aka root directory name")
@click.option("--app_id", default=0, help="set the app id")
def login(env, app_id):
    _login(env, app_id)


@cli.command()
@click.option("--environment", default=None, help="Name of the EDM environment")
@click.option(
    "--app",
    default="pycrunch",
    help="Application style to install. pycrunch, pyexperiment,...",
)
@click.option("--verbose/--no-verbose", default=False, help="Verbose output")
def edm(environment, app, verbose):
    _edm(environment, app, verbose)


@cli.command()
@click.option("--env", default="Pychron", help="Environment, aka root directory name")
@click.option("--use_ngx/--no-ngx", default=False, help="Install NGX")
@click.option(
    "--overwrite/--no-overwrite", default=False, help="Overwrite the file if it exists"
)
@click.option("--verbose/--no-verbose", default=False, help="Verbose output")
def setupfiles(env, use_ngx, overwrite, verbose):
    _setupfiles(env, use_ngx, overwrite, verbose)


@cli.command()
@click.option("--env", default="Pychron", help="Environment, aka root directory name")
@click.option("--use_ngx/--no-ngx", default=False, help="Install NGX")
@click.option(
    "--overwrite/--no-overwrite", default=False, help="Overwrite the file if it exists"
)
@click.option("--verbose/--no-verbose", default=False, help="Verbose output")
def scripts(env, use_ngx, overwrite, verbose):
    _scripts(env, use_ngx, overwrite, verbose)


@cli.command()
@click.option("--env", default="Pychron", help="Environment, aka root directory name")
@click.option(
    "--org",
    default="NMGRL",
    help="Github organization for storing laboratory files such as Plot Options",
)
@click.option("--use_ngx/--no-ngx", default=False, help="Install NGX")
@click.option(
    "--overwrite/--no-overwrite", default=False, help="Overwrite the file if it exists"
)
@click.option("--verbose/--no-verbose", default=False, help="Verbose output")
def buildenv(env, org, use_ngx, overwrite, verbose):
    _init(env, org, use_ngx, overwrite, verbose)
    _setupfiles(env, use_ngx, overwrite, verbose)
    _scripts(env, use_ngx, overwrite, verbose)


@cli.command()
@click.option("--fork", help="Name of the pychron fork to clone")
@click.option("--branch", default="dev/dr", help="Name of the pychron fork to clone")
@click.option("--app_id", default=0, help="set the app id")
def code(fork, branch, app_id):
    _code(fork, branch, app_id)


@cli.command()
@click.option("--conda/--no-conda", default=False, help="Use the conda package manager")
@click.option("--environment", default="pychron", help="Python environment name")
@click.option("--app", default="pycrunch", help="application name")
@click.option("--org", default="PychronLabsLLC", help="Github organization")
@click.option("--app_id", default=0, help="set the app id")
@click.option("--login/--no-login", default=0, help="show login window at startup")
@click.option(
    "--massspec_db_version", "msv", default=16, help="massspec database version"
)
@click.option("--output", default="pychron_launcher.sh", help="Output path")
@click.option(
    "--overwrite/--no-overwrite", default=False, help="Overwrite the file if it exists"
)
@click.option("--verbose/--no-verbose", default=False, help="Verbose output")
def launcher(
    conda, environment, app, org, app_id, login, msv, output, overwrite, verbose
):
    _launcher(
        conda, environment, app, org, app_id, login, msv, output, overwrite, verbose
    )


@cli.command()
@click.option("--env", default="Pychron", help="Environment, aka root directory name")
@click.option(
    "--org",
    default="NMGRL",
    help="Github organization for storing laboratory files such as Plot Options",
)
@click.option("--use_ngx/--no-ngx", default=False, help="Install NGX")
@click.option(
    "--overwrite/--no-overwrite", default=False, help="Overwrite the file if it exists"
)
@click.option("--verbose/--no-verbose", default=False, help="Verbose output")
def init(env, org, use_ngx, overwrite, verbose):
    _init(env, org, use_ngx, overwrite, verbose)


@cli.command()
@click.option("--env", default="Pychron", help="Environment, aka root directory name")
@click.option(
    "--overwrite/--no-overwrite", default=False, help="Overwrite the file if it exists"
)
def email(env, overwrite):
    _email(env, overwrite)


@cli.command()
@click.argument("name")
@click.option("--env", default="Pychron", help="Environment, aka root directory name")
@click.option(
    "--overwrite/--no-overwrite", default=False, help="Overwrite the file if it exists"
)
def makefile(name, env, overwrite):
    """ """
    _makefile(name, env, overwrite)


@cli.command()
@click.argument(
    "kind", type=click.Choice(["ngx", "argus", "helix"], case_sensitive=False)
)
@click.option("--env", default="Pychron", help="Environment, aka root directory name")
@click.option(
    "--overwrite/--no-overwrite", default=False, help="Overwrite the file if it exists"
)
def spectrometer_init(kind, env, overwrite):
    """ """
    _spectrometer_init(kind, env, overwrite)


@cli.command()
@click.argument("name", default="MetaData")
@click.option("--env", default="Pychron", help="Environment, aka root directory name")
@click.option(
    "--overwrite/--no-overwrite", default=False, help="Overwrite the file if it exists"
)
def metarepo(name, env, overwrite):
    """ """
    _metarepo(name, env, overwrite)


if __name__ == "__main__":
    cli()
# ============= EOF =============================================
